@@ -27,11 +27,11 @@ RDF::LinkedData::ProviderRole - Role providing important functionality for Linke
 
 =head1 VERSION
 
-Version 0.15_2
+Version 0.20
 
 =cut
 
-our $VERSION = '0.15_2';
+our $VERSION = '0.20';
 
 
 =head1 SYNOPSIS
@@ -74,13 +74,14 @@ very much in flux, and may change without warning.
 
 =over
 
-=item C<< new ( store => $store, model => $model, base => $base, headers_in => $headers_in ) >>
+=item C<< new ( store => $store, model => $model, base_uri => $base_uri, request => $request ) >>
 
 Creates a new handler object based on named parameters, given a store
-config (which is a hashref that can be passed to
-L<RDF::Trine::Store>->new_with_config) or model and a base
-URI. Optionally, you may pass a Apache request object, and you will
-need to pass a L<HTTP::Headers> object if you plan to call C<content>.
+config (recommended usage is to pass a hashref of the type that can be
+passed to L<RDF::Trine::Store>->new_with_config, but a simple string
+can also be used) or model and a base URI. Optionally, you may pass a
+Apache request object, and you will need to pass a L<HTTP::Headers>
+object if you plan to call C<content>.
 
 =cut
 
@@ -92,33 +93,41 @@ sub BUILD {
 	  my $i = 0;
 	  foreach my $source (@{$self->store->{sources}}) {
 	    unless ($source->{base_uri}) {
-	      ${$self->store->{sources}}[$i]->{base_uri} = $self->base;
+	      ${$self->store->{sources}}[$i]->{base_uri} = $self->base_uri;
 	    }
 	    $i++;
 	  }
-	  my $store	= RDF::Trine::Store->new_with_config( $self->store );
+	  my $store	= RDF::Trine::Store->new( $self->store );
 	  $self->model(RDF::Trine::Model->new( $store ));
 	}
 
         throw Error -text => "No valid RDF::Trine::Model, need either a store config hashref or a model." unless ($self->model);
 
+ 	if ($self->endpoint_config) {
+ 	  eval { require RDF::Endpoint; };
+ 	  if ($@) {
+ 	    throw Error -text => "RDF::Endpoint not installed. Please install or remove its configuration.";
+ 	  }
+ 	  $self->endpoint(RDF::Endpoint->new($self->model, $self->endpoint_config));
+ 	}
 }
 
+has endpoint_config => (is => 'ro', isa => 'HashRef' );
+
+has endpoint => (is => 'rw', isa => 'RDF::Endpoint', default => undef );
 
 has store => (is => 'rw', isa => 'HashRef' );
 
 
-=item C<< headers_in ( [ $headers ] ) >>
+=item C<< request ( [ $headers ] ) >>
 
 Returns the L<HTTP::Headers> object if it exists or sets it if a L<HTTP::Headers> object is given as parameter.
 
 =cut
 
-has headers_in => ( is => 'rw', isa => 'HTTP::Headers', builder => '_build_headers_in');
+has request => ( is => 'rw', isa => 'Plack::Request');
+
 
-sub _build_headers_in {
-    return HTTP::Headers->new() ;
-}
 
 =item C<< helper_properties (  ) >>
 
@@ -203,8 +212,8 @@ sub content {
     my %output;
     if ($type eq 'data') {
         $self->{_type} = 'data';
-        my ($type, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $self->headers_in,
-                                                           base => $self->base,
+        my ($type, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $self->request->headers,
+                                                           base => $self->base_uri,
                                                            namespaces => $self->namespaces);
         my $iter = $model->bounded_description($node);
         $output{content_type} = $type;
@@ -214,7 +223,8 @@ sub content {
         my $preds = $self->helper_properties;
         my $title		= $preds->title( $node );
         my $desc		= $preds->description( $node );
-        my $description	= sprintf( "<table>%s</table>\n", join("\n\t\t", map { sprintf( '<tr><td>%s</td><td>%s</td></tr>', @$_ ) } @$desc) );
+        my $description	= sprintf('<table about="'. $node->uri_value  .'"' .">%s</table>\n", 
+				  join("\n\t\t", map { sprintf( '<tr><td>%s</td><td>%s</td></tr>', @$_ ) } @$desc) );
         $output{content_type} = 'text/html';
         $output{body} =<<"END";
 <?xml version="1.0"?>
@@ -251,13 +261,13 @@ Returns or sets the RDF::Trine::Model object.
 
 has model => (is => 'rw', isa => 'RDF::Trine::Model');
 
-=item C<< base >>
+=item C<< base_uri >>
 
 Returns or sets the base URI for this handler.
 
 =cut
 
-has base => (is => 'rw', isa => 'Str' );
+has base_uri => (is => 'rw', isa => 'Str' );
 
 
 =item C<< response ( $uri ) >>
@@ -268,9 +278,17 @@ response object.
 =cut
 
 sub response {
-    my ($self, $uri) = @_;
+    my $self = shift;
+    my $uri = URI->new(shift);
     my $response = Plack::Response->new;
 
+    my $headers_in = $self->request->headers;
+    my $endpoint_path = $self->endpoint_config->{endpoint_path} || '/sparql';
+
+    if(defined($self->endpoint) && ($uri->path eq $endpoint_path)) {
+      return $self->endpoint->run( $self->request );
+    }
+
     my $type = $self->type;
     $self->type('');
     my $node = $self->my_node($uri);
@@ -288,11 +306,11 @@ sub response {
             }
 
             $self->logger->debug("Will render '$type' page ");
-            if ($self->headers_in->can('header') && $self->headers_in->header('Accept')) {
-                $self->logger->debug('Found Accept header: ' . $self->headers_in->header('Accept'));
+            if ($headers_in->can('header') && $headers_in->header('Accept')) {
+                $self->logger->debug('Found Accept header: ' . $headers_in->header('Accept'));
             } else {
-                $self->headers_in(HTTP::Headers->new('Accept' => 'application/rdf+xml'));
-                $self->logger->warn('Setting Accept header: ' . $self->headers_in->header('Accept'));
+                $headers_in->header(HTTP::Headers->new('Accept' => 'application/rdf+xml'));
+                $self->logger->warn('Setting Accept header: ' . $headers_in->header('Accept'));
             }
             $response->status(200);
             my $content = $self->content($node, $type);
@@ -301,17 +319,26 @@ sub response {
             $response->content($content->{body});
         } else {
             $response->status(303);
-            my $ct;
+            my ($ct, $s);
             eval {
-                ($ct) = RDF::Trine::Serializer->negotiate('request_headers' => $self->headers_in,
-                                                          base => $self->base,
-                                                          namespaces => $self->namespaces);
-            };
+                ($ct, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $headers_in,
+                                                          base => $self->base_uri,
+                                                          namespaces => $self->namespaces,
+							  extend => {
+								     'text/html'	=> 'html',
+								     'application/xhtml+xml' => 'html'
+								    }
+							  )
+	      };
+            $self->logger->debug("Got $ct content type");
             if ($@) {
-                $ct = 'text/html'; # Set it to HTML for now
+	      $response->status(406);
+	      $response->headers->content_type('text/plain');
+	      $response->body('HTTP 406: No serialization available any specified content type');
+	      return $response;
             }
             my $newurl = $uri . '/data';
-            unless ($ct =~ /rdf|turtle/) {
+            unless ($s->isa('RDF::Trine::Serializer')) {
                 my $preds = $self->helper_properties;
                 $newurl = $preds->page($node);
             }
@@ -326,7 +353,7 @@ sub response {
         $response->headers->content_type('text/plain');
         $response->body('HTTP 404: Unknown resource');
         return $response;
-    }
+      }
     # We should never get here.
     $response->status(500);
     $response->headers->content_type('text/plain');
@@ -378,7 +405,7 @@ L<http://lists.perlrdf.org/listinfo/dev>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2010 Gregory Todd Williams and ABC Startsiden AS.
+Copyright 2010-2011 Kjetil Kjernsmo, Gregory Todd Williams and ABC Startsiden AS.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
